from bdaserviceutils import get_args, get_cmd_arg
from .generic_service import GenericService
from alidaargparser import get_asset_property

try: 
    import pysparkutilities
except ImportError:
    import pip
    pip.main(['install', "pyspark-utilities"])

from pysparkutilities.spark_initializer import spark_initializer
from pysparkutilities import ds_initializer, ModelStorage

class SparkService(GenericService):
    def __init__(self, parser, additional_config=[]):
        super().__init__(parser)
        self._args = get_args()
        print(self._args)

        #additional_config = additional_config.append(('spark.jars.packages', 'io.prestosql:presto-jdbc:350'))
        self.spark = spark_initializer("Test-model", self._args, additional_config=additional_config)

    def get_dataset(self, name="input_dataset"):
        data = ds_initializer.load_dataset(sc=self.spark, name=name, read_all=False)
        return data

    def save_dataset(self, dataset, name="output_dataset"):
        ds_initializer.save_dataset(sc=self.spark,name=name, df=dataset)#, output_dest=self._args['output-dataset'])

    def download_model(self, name='input_model'):
        modelStorage = ModelStorage(spark=self.spark, name=name)
        return modelStorage.load_model()

    def upload_model(self, model, name='output_model'):
        if isinstance(model, str):
            super().upload_model(model, name)
        else:       
            outmod = ModelStorage(spark=self.spark, name=name)
            outmod.save_model(model=model)
            
    def get_input_datasets(self):
        datasets = {}
        for dataset in self.parser.input_datasets:
            datasets[dataset.name] = self.get_dataset(dataset.name)
        return next(iter(datasets.values()))
    


from abc import abstractclassmethod
from .streaming_service import StreamingService
from bdaserviceutils import get_kafka_binder_brokers, get_input_channel, get_output_channel
from kafka import KafkaConsumer, KafkaProducer
from abc import ABC, abstractclassmethod
import json


class ProcessorService(ABC, StreamingService):

    alida_service_mode = "processor"
    
    def __init__(self, parser):
        super().__init__(parser)
        self.consumer = self.get_consumer()
        self.producer = self.get_producer()

    def run(self):
        for message in self.consumer.read_message():
            self.on_message(message=message)

    @abstractclassmethod
    def on_message(self, message):
        pass

    def send_message(self, message):
        self.producer.send_message(message)



import os
import logging
from bdaserviceutils import get_args
from dsioutilities import Dataset
from .utils import packages_are_installed
from alidaargparser.assets_utils import get_asset_properties

if packages_are_installed(['fileioutilities']):
    from fileioutilities.file_io import FileIO
else:
    logging.warning("Please install fileioutilities (https://pypi.org/project/file-io-utilities) if you want to use models in your service.")

class GenericService():
    def __init__(self, parser):
        super().__init__()
        
        # Raw args
        self._args = get_args()
        
        self.parser = parser
        
        # Parsed args
        self.args = self.parser.parse_args()
        
        # Set ports to expose
        self.ports = self.parser.parse_ports()

        # If only one port is present, set it as self.port
        if len(self.ports.values())==1:
            self.port = list(self.ports.values())[0]

    def get_assets_info(self):
        
        info = {"datasets": {}, "models": {}}

        for dataset in self.parser.input_datasets:
            props = get_asset_properties(dataset.name)
            info['datasets'][dataset.name] = props

        for dataset in self.parser.output_datasets:
            props = get_asset_properties(dataset.name)
            info['datasets'][dataset.name] = props

        for model in self.parser.input_models:
            props = get_asset_properties(model.name)
            info['models'][model.name] = props

        for model in self.parser.output_models:
            props = get_asset_properties(model.name)
            info['models'][model.name] = props


        return info

    def get_datasets_full(self):
        datasets = {}
        for dataset in self.parser.input_datasets:
            datasets[dataset.name] = self.get_dataset(dataset.name)
        return datasets

    def get_input_datasets(self):
        datasets = {}
        for dataset in self.parser.input_datasets:
            datasets[dataset.name] = self.get_dataset(dataset.name)
        return datasets
    
    def get_output_datasets(self):
        datasets = {}
        for dataset in self.parser.output_datasets:
            datasets[dataset.name] = self.get_dataset(dataset.name)
        return datasets


    def get_datasets(self):
        datasets = self.get_datasets_full()

        # If only one dataset, return it directly
        if len(datasets) == 1:
            return datasets[next(iter(datasets))]

        return datasets

    def save_datasets(self, datasets):
        if isinstance(datasets, dict):
            for dataset in self.parser.output_datasets:
                self.save_dataset(datasets[dataset.name], dataset.name)
        # If only one dataset a dictionary is not expected
        else:
            self.save_dataset(datasets, self.parser.output_datasets[0].name)


    def get_models_full(self):
        models = {}
        for model in self.parser.input_models:
            models[model.name] = self.download_model(model.name)

        return models


    def download_models(self):
        models = self.get_models_full()
        
        # If only one model, return it directly
        if len(models) == 1:
            return models[next(iter(models))]
        
        return models

    def upload_models(self, models):     
        if isinstance(models, dict):
            for model in self.parser.output_models:
                self.upload_model(models[model.name], model.name)   
        # If only one dataset a dictionary is not expected
        else:
            self.upload_model(models, self.parser.output_models[0].name)

    def get_dataset(self, name="input-dataset"):
        return Dataset(name, dataset_type="object")

    def get_output_dataset(self, name="output-dataset"):
        return self.get_dataset(name)

    def save_dataset(self, dataset, name="output-dataset"):
        pass

    def download_model(self, name='input_model'):
        path = os.path.join(".", "model")
        FileIO(name=name).download(local_path=path)
        return path

    def upload_model(self, path, name='output_model'):
        fileIO = FileIO(name=name)
        fileIO.upload(local_path=path)



from abc import abstractclassmethod
from .streaming_service import StreamingService
from bdaserviceutils import get_kafka_binder_brokers, get_input_channel
from kafka import KafkaConsumer
from abc import ABC, abstractclassmethod


class SinkService(ABC, StreamingService):
    
    alida_service_mode = "sink"
    
    def __init__(self, parser):
        super().__init__(parser)
        self.consumer = self.get_consumer()

    def run(self):
        for message in self.consumer.read_message():
            self.on_message(message=message)

    @abstractclassmethod
    def on_message(self, message):
        pass



from .utils import packages_are_installed
import logging

from .pandas_service import PandasService
from .io import IO
from .processor_service import ProcessorService
from .source_service import SourceService
from .sink_service import SinkService
from .generic_service import GenericService



from .generic_service import GenericService
from alidadataset.dataset import load as load_dataset

class StreamingService(GenericService):

    def __init__(self, parser):
        super().__init__(parser)

    def get_consumer(self):
        return load_dataset(name="input_dataset", load_as="streaming_input")
    
    def get_producer(self):
        return load_dataset(name="output_dataset", load_as="streaming_output")



from .generic_service import GenericService
import pandas as pd
from dsioutilities import Dataset
from alidaargparser import get_asset_property

class PandasService(GenericService):
    def __init__(self, parser):
        super().__init__(parser)

    def get_dataset(self, name="input-dataset"):
        input_dataset = Dataset(name, dataset_type="tabular")
        
        # If list of csv concatenate them
        if isinstance(input_dataset.get_path(), list):
            df = pd.concat([pd.read_csv(filename) for filename in input_dataset.get_path()])
        else:
            df = pd.read_csv(input_dataset.get_path())
        
        # TODO select by type
        # Select specified columns
        input_cols = get_asset_property(name, "input_columns")
        if input_cols is not None and input_cols != "ANY" and input_cols != "NUMBER" and input_cols != "*":
            df = df[input_cols.split(",")]
        return df
    

    def get_input_datasets(self):
        datasets = {}
        for dataset in self.parser.input_datasets:
            datasets[dataset.name] = self.get_dataset(dataset.name)
        return next(iter(datasets.values()))
    

    def save_dataset(self, dataset: pd.DataFrame, name="output-dataset"):
        dataset.to_csv(Dataset(name, dataset_type="tabular").get_path(), index=False)
